@@ -1,15 +1,15 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2007 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2007,2008 -- leonerd@leonerd.org.uk
 
 package IO::Async::DetachedCode;
 
 use strict;
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
-use IO::Async::Buffer;
+use IO::Async::Stream;
 
 use Carp;
 
@@ -19,19 +19,18 @@ use constant LENGTH_OF_I => length( pack( "I", 0 ) );
 
 =head1 NAME
 
-C<IO::Async::DetachedCode> - a class that allows a block of code to execute
-asynchronously in detached child processes
+C<IO::Async::DetachedCode> - execute code asynchronously in child processes
 
 =head1 SYNOPSIS
 
-Usually this object would be constructed indirectly, via an C<IO::Async::Set>:
+Usually this object would be constructed indirectly, via an C<IO::Async::Loop>:
 
- use IO::Async::Set::...;
- my $set = IO::Async::Set::...
+ use IO::Async::Loop::...;
+ my $loop = IO::Async::Loop::...
 
- $set->enable_childmanager;
+ $loop->enable_childmanager;
 
- my $code = $set->detach_code(
+ my $code = $loop->detach_code(
     code => sub {
        my ( $number ) = @_;
        return is_prime( $number );
@@ -49,15 +48,15 @@ Usually this object would be constructed indirectly, via an C<IO::Async::Set>:
     },
  );
 
- $set->loop_forever;
+ $loop->loop_forever;
 
 It can also be used directly. In this case, extra effort must be taken to pass
-an C<IO::Async::Set> object:
+an C<IO::Async::Loop> object:
 
- my $set = IO::Async::Set::...
+ my $loop = IO::Async::Loop::...
 
  my $code = IO::Async::DetachedCode->new(
-    set => $set,
+    loop => $loop,
     code => sub { ... },
  );
 
@@ -114,9 +113,9 @@ The C<%params> hash takes the following keys:
 
 =over 8
 
-=item set => IO::Async::Set
+=item loop => IO::Async::Loop
 
-A reference to an C<IO::Async::Set> object. The set must have the child
+A reference to an C<IO::Async::Loop> object. The loop must have the child
 manager enabled.
 
 =item code => CODE
@@ -184,7 +183,7 @@ sub new
    my $class = shift;
    my ( %params ) = @_;
 
-   my $set = delete $params{set} or croak "Expected a 'set'";
+   my $loop = delete $params{loop} or croak "Expected a 'loop'";
 
    my $code = delete $params{code};
    ref $code eq "CODE" or croak "Expected a CODE reference as 'code'";
@@ -216,7 +215,7 @@ sub new
    my $self = bless {
       next_id     => 0,
       code        => $code,
-      set         => $set,
+      loop        => $loop,
       streamtype  => $streamtype,
       marshaller  => $marshaller,
       workers     => $workers,
@@ -238,7 +237,7 @@ sub _detach_child
 
    # The inner object needs references to some members of the outer object
    my $inner = {
-      set            => $self->{set},
+      loop           => $self->{loop},
       result_handler => {},
       marshaller     => $self->{marshaller},
       busy           => 0,
@@ -264,9 +263,9 @@ sub _detach_child
       pipe( $myread, $childwrite ) or croak "Cannot pipe() - $!";
    }
 
-   my $set = $inner->{set};
+   my $loop = $inner->{loop};
 
-   my $kid = $set->detach_child(
+   my $kid = $loop->detach_child(
       code => sub { 
          foreach( 0 .. IO::Async::ChildManager::OPEN_MAX_FD() ) {
             next if $_ == 2;
@@ -286,16 +285,16 @@ sub _detach_child
    close( $childread );
    close( $childwrite );
 
-   my $iobuffer = IO::Async::Buffer->new(
+   my $iostream = IO::Async::Stream->new(
       read_handle  => $myread,
       write_handle => $mywrite,
 
       on_read => sub { _socket_incoming( $inner, $_[1], $_[2] ) },
    );
 
-   $inner->{iobuffer} = $iobuffer;
+   $inner->{iostream} = $iostream;
 
-   $set->add( $iobuffer );
+   $loop->add( $iostream );
 
    push @{ $self->{inners} }, $inner;
 
@@ -429,9 +428,11 @@ sub shutdown
    my $self = shift;
 
    foreach my $inner ( @{ $self->{inners} } ) {
-      if( defined $inner->{iobuffer} ) {
-         $inner->{set}->remove( $inner->{iobuffer} );
-         undef $inner->{iobuffer};
+      # This is called from DESTROY, so all sorts of craziness might have
+      # happened. We need to be extra-paranoid.
+      if( defined $inner->{iostream} and defined $inner->{loop} ) {
+         $inner->{loop}->remove( $inner->{iostream} );
+         undef $inner->{iostream};
       }
 
       my $handlermap = $inner->{result_handler};
@@ -441,6 +442,8 @@ sub shutdown
          delete $handlermap->{$id};
       }
    }
+
+   @{ $self->{inners} } = ();
 }
 
 =head2 $n_workers = $code->workers
@@ -470,7 +473,7 @@ sub _send_request
    my $handlermap = $inner->{result_handler};
    $handlermap->{$callid} = $on_result;
 
-   $inner->{iobuffer}->write( pack( "I", length $request ) . $request );
+   $inner->{iostream}->write( pack( "I", length $request ) . $request );
    $inner->{busy} = 1;
 }
 
@@ -484,8 +487,8 @@ sub _socket_incoming
    if( $closed ) {
       _child_error( $inner, 'closed' );
 
-      $inner->{set}->remove( $inner->{iobuffer} );
-      undef $inner->{iobuffer};
+      $inner->{loop}->remove( $inner->{iostream} );
+      undef $inner->{iostream};
 
       return 0;
    }
@@ -519,8 +522,8 @@ sub _socket_incoming
       if( $inner->{exit_on_die} ) {
          _child_error( $inner, 'die' );
 
-         $inner->{set}->remove( $inner->{iobuffer} );
-         undef $inner->{iobuffer};
+         $inner->{loop}->remove( $inner->{iostream} );
+         undef $inner->{iostream};
       }
    }
 
